@@ -11,6 +11,8 @@ from __future__ import annotations
 import sys
 import uuid
 import os
+import fnmatch
+
 import asyncio
 from os.path import basename
 from socket import gethostname
@@ -35,6 +37,7 @@ class Proxy():
    
     __commands = "__commands"
     __commands_key = "help"
+    __pull_commands_task = "_pull_commands_task"
     
     __amqpc = None
 
@@ -74,29 +77,24 @@ class Proxy():
     def setDefaultAmqpc(amqpc):
         Proxy.__amqpc = amqpc
 
-    #def __getattr__(self, attr):
-        #if attr != "_pull_commands_task":
-            #try:
-                #super(Proxy, self).__getattribute__(self, "ping")
+    def __getattr__(self, attr):
+        # order is important !
+        if attr != Proxy.__pull_commands_task and hasattr(self, Proxy.__pull_commands_task):
+            return partial(self.call_command, attr)
+        return super(Proxy, self).__getattribute__(attr)
 
-            #except AttributeError:
-                #self.amqpc.log.warning(f"actor {self.actor} {attr} currently not reachable.")
-                #setattr(self, attr, partial(self.call_command, attr))
-
-        #return super(Proxy, self).__getattribute__(attr)
-##        return self.__getattribute__(attr)
 
     async def __delattr_pull_commands_task(self, cancel=False):
         lock = asyncio.Lock()
         async with lock:
-            if hasattr(self, "_pull_commands_task"):
+            if hasattr(self, Proxy.__pull_commands_task):
                 if cancel:
                     self._pull_commands_task.cancel()
                     try:
                         await self._pull_commands_task
                     except asyncio.exceptions.CancelledError as ex:
                         self.amqpc.log.debug(f"error {ex}")
-                delattr(self ,"_pull_commands_task")
+                delattr(self ,Proxy.__pull_commands_task)
         
 
     async def _pull_commands(self, delay = 0, attempts = 1):
@@ -119,12 +117,12 @@ class Proxy():
             except Exception as ex:
                 if not delay:
                     self.amqpc.log.warning(f"actor {self.actor} currently not reachable.")
-                if not hasattr(self, "_pull_commands_task"):
-                    self.amqpc.log.debug(f"attempt delayed connection as background task.")
+                if not hasattr(self, Proxy.__pull_commands_task):
+                    self.amqpc.log.debug(f"actor {self.actor} connect as background task.")
                     self._pull_commands_task = self.amqpc.loop.create_task(self._pull_commands(Proxy.pull_commands_delay, Proxy.pull_commands_attempts))
                     return    
 
-        self.amqpc.log.debug(f"stop delayed connection as background task.")
+        self.amqpc.log.debug(f"actor {self.actor} connect attempts stopped.")
         await self.__delattr_pull_commands_task()
 
 
@@ -210,7 +208,7 @@ async def invoke(*cmds, return_exceptions:Bool=False):
     return ret
 
 
-def unpack(ret, *keys):
+def unpack(data, *keys):
     """ unpacks every parameter from the message of the finish reply or list of replies.
 
         Pythons list unpacking mechanism PEP3132 can be used to assign the value(s)
@@ -243,26 +241,27 @@ def unpack(ret, *keys):
         return only the parameters from keys
     """
 
-    if len(ret) == 0:
+    if len(data) == 0:
         return
-    
-    if isinstance(ret, list):
+
+    if isinstance(data, list):
         if len(keys) > 0:
-            rkeys = [k for r in ret for k in list(r.keys())]
-            bkeys = [k for k in keys if k not in rkeys]
+            rkeys = [k for r in data for k in list(r.keys())]
+            bkeys = [k for k in keys if not fnmatch.filter(rkeys, k)]
             if bkeys:
                 raise KeyError(bkeys)
 
-            return [d[k] for d in ret for k in keys if k in d]
+            flt_data = [d[fn] for k in keys for d in data for fn in fnmatch.filter(d, k)] #[d[k] for d in data for k in keys if k in d]
+            return flt_data if len(flt_data) > 1 else flt_data[0]
         else:
-            return [val for d in ret for val in list(d.values())]
+            return [val for d in data for val in list(d.values())] if len(data) > 1 else data[0]
 
-    if len(ret) == 1:
-        return list(ret.values())[0]
+    if len(data) == 1:
+        return list(data.values())[0]
     elif len(keys) > 0:
-        return [ret[k] for k in keys]
-    else:
-        return list(ret.values())
+        flt_data = [data[fn] for k in keys for fn in fnmatch.filter(data, k)] #[data[k] for k in keys]
+        return flt_data if len(flt_data) > 1 else flt_data[0]
+    return list(data.values())
 
 
 def flatten(d: MutableMapping, parent_key: str = '', sep: str = '.'):
